`timescale 1ns / 1ps
//////////////////////////////////////////////////////////////////////////////////
// Company: 
// Engineer: 
// 
// Create Date: 11/25/2023 11:27:26 AM
// Design Name: 
// Module Name: floating_point_fsm_test
// Project Name: 
// Target Devices: 
// Tool Versions: 
// Description: 
// 
// Dependencies: 
// 
// Revision:
// Revision 0.01 - File Created
// Additional Comments:
// 
//////////////////////////////////////////////////////////////////////////////////


module floating_point_fsm_test();
    reg clk;
    reg rst;
    reg en;
    reg [11:0] a;
    reg [11:0] b;
    wire done;
    wire [11:0] out;
    
    reg [11:0] act_out, exp_out;
    integer err_cnt = 0;
    
    //DUT instantiation
    floating_point_fsm DUT (
        .clk(clk),
        .rst(rst),
        .en(en),
        .a(a),
        .b(b),
        .done(done),
        .out(out)
    );
    
    //Stimuli generation
    initial begin
         clk = 0;
         rst = 1;
         en  = 0;
         a   = 0;
         b   = 0;
         
         //Create Stimuli Groups
         #100;
         rst = 0;
         
         //----1st pair simple test
         @(posedge clk);    en = 1; a = 12'b0_0000_0000000; b = 12'b0_0000_0000000;
         @(posedge clk);    en = 0; a = 12'b0_1011_0010110; b = 12'b0_1000_0000101; 
         @(posedge clk); #0 en = 0; a = 12'b0_0000_0000000; b = 12'b0_0000_0000000;
         
         //Expect the result
         @(posedge done);
            @(negedge clk);
                act_out = out;
                exp_out = 12'b0_1100_0011011;
                
                if(act_out !== exp_out) begin
                    err_cnt = err_cnt + 1;
                    $display("Wrong result: Act %b vs Exp %b", act_out, exp_out);
                end else begin
                    $display("1st set OK");
                end
          
          #100;
         
         //----2nd pair normalize
         @(posedge clk);    en = 1; a = 12'b0_0000_0000000; b = 12'b0_0000_0000000;
         @(posedge clk);    en = 0; a = 12'b0_1100_0101010; b = 12'b0_1001_1101110; 
         @(posedge clk); #0 en = 0; a = 12'b0_0000_0000000; b = 12'b0_0000_0000000;
         
         //Expect the result
         @(posedge done);
            @(negedge clk);
                act_out = out;
                exp_out = 12'b0_1111_0011110;
                
                if(act_out !== exp_out) begin
                    err_cnt = err_cnt + 1;
                    $display("Wrong result: Act %b vs Exp %b", act_out, exp_out);
                end else begin
                    $display("2nd set OK");
                end
          
          #100;
         
         
         //----3rd pair : sign and normalize
         @(posedge clk);    en = 1; a = 12'b0_0000_0000000; b = 12'b0_0000_0000000;
         @(posedge clk);    en = 0; a = 12'b1_1001_1011010; b = 12'b0_1010_0011001; 
         @(posedge clk); #0 en = 0; a = 12'b0_0000_0000000; b = 12'b0_0000_0000000;
         
         //Expect the result
         @(posedge done);
            @(negedge clk);
                act_out = out;
                exp_out = 12'b1_1101_0000010;
                
                if(act_out !== exp_out) begin
                    err_cnt = err_cnt + 1;
                    $display("Wrong result: Act %b vs Exp %b", act_out, exp_out);
                end else begin
                    $display("3rd set OK");
                end
          
          #100;
          
          //----4th pair : sign and normalize
         @(posedge clk);    en = 1; a = 12'b0_0000_0000000; b = 12'b0_0000_0000000;
         @(posedge clk);    en = 0; a = 12'b0_0001_0010100; b = 12'b1_1010_1000110; 
         @(posedge clk); #0 en = 0; a = 12'b0_0000_0000000; b = 12'b0_0000_0000000;
         
         //Expect the result
         @(posedge done);
            @(negedge clk);
                act_out = out;
                exp_out = 12'b1_0100_1100100;
                
                if(act_out !== exp_out) begin
                    err_cnt = err_cnt + 1;
                    $display("Wrong result: Act %b vs Exp %b", act_out, exp_out);
                end else begin
                    $display("4th set OK");
                end
          
          #100;
          
        if(err_cnt == 0)
            $display("Test ended succesfully");
        else
            $display("Test fail succesfully with %d errors", err_cnt);
         
    end
    
    always #10 clk = !clk;
    
endmodule
